@@ -3,7 +3,11 @@
 // Copyright (c) 2007-2012 Barend Gehrels, Amsterdam, the Netherlands.
 // Copyright (c) 2008-2012 Bruno Lalande, Paris, France.
 // Copyright (c) 2009-2012 Mateusz Loskot, London, UK.
-// Copyright (c) 2014 Adam Wulkiewicz, Lodz, Poland.
+// Copyright (c) 2014-2017 Adam Wulkiewicz, Lodz, Poland.
+
+// This file was modified by Oracle on 2017-2020.
+// Modifications copyright (c) 2017-2020 Oracle and/or its affiliates.
+// Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 
 // Parts of Boost.Geometry are redesigned from Geodan's Geographic Library
 // (geolib/GGL), copyright (c) 1995-2010 Geodan, Amsterdam, the Netherlands.
@@ -20,14 +24,15 @@
 #include <cstddef>
 #include <functional>
 
-#include <boost/mpl/assert.hpp>
-#include <boost/range.hpp>
-#include <boost/type_traits/remove_reference.hpp>
+#include <boost/range/begin.hpp>
+#include <boost/range/end.hpp>
+#include <boost/range/value_type.hpp>
 
 #include <boost/variant/apply_visitor.hpp>
 #include <boost/variant/static_visitor.hpp>
 #include <boost/variant/variant_fwd.hpp>
 
+#include <boost/geometry/algorithms/correct_closure.hpp>
 #include <boost/geometry/algorithms/detail/interior_iterator.hpp>
 
 #include <boost/geometry/core/closure.hpp>
@@ -41,7 +46,6 @@
 #include <boost/geometry/geometries/concepts/check.hpp>
 
 #include <boost/geometry/algorithms/area.hpp>
-#include <boost/geometry/algorithms/disjoint.hpp>
 #include <boost/geometry/algorithms/detail/multi_modify.hpp>
 #include <boost/geometry/util/order_as_direction.hpp>
 
@@ -61,7 +65,8 @@ namespace detail { namespace correct
 template <typename Geometry>
 struct correct_nop
 {
-    static inline void apply(Geometry& )
+    template <typename Strategy>
+    static inline void apply(Geometry& , Strategy const& )
     {}
 };
 
@@ -104,8 +109,8 @@ struct correct_box_loop<Box, DimensionCount, DimensionCount>
 template <typename Box>
 struct correct_box
 {
-
-    static inline void apply(Box& box)
+    template <typename Strategy>
+    static inline void apply(Box& box, Strategy const& )
     {
         // Currently only for Cartesian coordinates
         // (or spherical without crossing dateline)
@@ -119,49 +124,30 @@ struct correct_box
 
 
 // Close a ring, if not closed
-template <typename Ring, typename Predicate>
+template <typename Ring, template <typename> class Predicate>
 struct correct_ring
 {
     typedef typename point_type<Ring>::type point_type;
     typedef typename coordinate_type<Ring>::type coordinate_type;
 
-    typedef typename strategy::area::services::default_strategy
-        <
-            typename cs_tag<point_type>::type,
-            point_type
-        >::type strategy_type;
-
-    typedef detail::area::ring_area
-            <
-                order_as_direction<geometry::point_order<Ring>::value>::value,
-                geometry::closure<Ring>::value
-            > ring_area_type;
-
-
-    static inline void apply(Ring& r)
+    template <typename Strategy>
+    static inline void apply(Ring& r, Strategy const& strategy)
     {
-        // Check close-ness
-        if (boost::size(r) > 2)
-        {
-            // check if closed, if not, close it
-            bool const disjoint = geometry::disjoint(*boost::begin(r), *(boost::end(r) - 1));
-            closure_selector const s = geometry::closure<Ring>::value;
+        // Correct closure if necessary
+        detail::correct_closure::close_or_open_ring<Ring>::apply(r);
 
-            if (disjoint && (s == closed))
-            {
-                geometry::append(r, *boost::begin(r));
-            }
-            if (! disjoint && s != closed)
-            {
-                // Open it by removing last point
-                geometry::traits::resize<Ring>::apply(r, boost::size(r) - 1);
-            }
-        }
         // Check area
-        Predicate predicate;
-        typedef typename default_area_result<Ring>::type area_result_type;
-        area_result_type const zero = area_result_type();
-        if (predicate(ring_area_type::apply(r, strategy_type()), zero))
+        typedef typename area_result<Ring, Strategy>::type area_result_type;
+        Predicate<area_result_type> predicate;
+        area_result_type const zero = 0;
+        if (predicate(detail::area::ring_area::apply(
+                        r,
+                        // TEMP - in the future (umbrella) strategy will be passed
+                        geometry::strategies::area::services::strategy_converter
+                            <
+                                Strategy
+                            >::get(strategy)),
+                      zero))
         {
             std::reverse(boost::begin(r), boost::end(r));
         }
@@ -174,15 +160,15 @@ template <typename Polygon>
 struct correct_polygon
 {
     typedef typename ring_type<Polygon>::type ring_type;
-    typedef typename default_area_result<Polygon>::type area_result_type;
-
-    static inline void apply(Polygon& poly)
+    
+    template <typename Strategy>
+    static inline void apply(Polygon& poly, Strategy const& strategy)
     {
         correct_ring
             <
                 ring_type,
-                std::less<area_result_type>
-            >::apply(exterior_ring(poly));
+                std::less
+            >::apply(exterior_ring(poly), strategy);
 
         typename interior_return_type<Polygon>::type
             rings = interior_rings(poly);
@@ -192,8 +178,8 @@ struct correct_polygon
             correct_ring
                 <
                     ring_type,
-                    std::greater<area_result_type>
-                >::apply(*it);
+                    std::greater
+                >::apply(*it, strategy);
         }
     }
 };
@@ -237,7 +223,7 @@ struct correct<Ring, ring_tag>
     : detail::correct::correct_ring
         <
             Ring,
-            std::less<typename default_area_result<Ring>::type>
+            std::less
         >
 {};
 
@@ -281,29 +267,36 @@ namespace resolve_variant {
 template <typename Geometry>
 struct correct
 {
-    static inline void apply(Geometry& geometry)
+    template <typename Strategy>
+    static inline void apply(Geometry& geometry, Strategy const& strategy)
     {
         concepts::check<Geometry const>();
-        dispatch::correct<Geometry>::apply(geometry);
+        dispatch::correct<Geometry>::apply(geometry, strategy);
     }
 };
 
 template <BOOST_VARIANT_ENUM_PARAMS(typename T)>
 struct correct<boost::variant<BOOST_VARIANT_ENUM_PARAMS(T)> >
 {
+    template <typename Strategy>
     struct visitor: boost::static_visitor<void>
     {
+        Strategy const& m_strategy;
+
+        visitor(Strategy const& strategy): m_strategy(strategy) {}
+
         template <typename Geometry>
         void operator()(Geometry& geometry) const
         {
-            correct<Geometry>::apply(geometry);
+            correct<Geometry>::apply(geometry, m_strategy);
         }
     };
 
+    template <typename Strategy>
     static inline void
-    apply(boost::variant<BOOST_VARIANT_ENUM_PARAMS(T)>& geometry)
+    apply(boost::variant<BOOST_VARIANT_ENUM_PARAMS(T)>& geometry, Strategy const& strategy)
     {
-        boost::apply_visitor(visitor(), geometry);
+        boost::apply_visitor(visitor<Strategy>(strategy), geometry);
     }
 };
 
@@ -325,7 +318,36 @@ struct correct<boost::variant<BOOST_VARIANT_ENUM_PARAMS(T)> >
 template <typename Geometry>
 inline void correct(Geometry& geometry)
 {
-    resolve_variant::correct<Geometry>::apply(geometry);
+    typedef typename point_type<Geometry>::type point_type;
+
+    typedef typename strategy::area::services::default_strategy
+        <
+            typename cs_tag<point_type>::type
+        >::type strategy_type;
+
+    resolve_variant::correct<Geometry>::apply(geometry, strategy_type());
+}
+
+/*!
+\brief Corrects a geometry
+\details Corrects a geometry: all rings which are wrongly oriented with respect
+    to their expected orientation are reversed. To all rings which do not have a
+    closing point and are typed as they should have one, the first point is
+    appended. Also boxes can be corrected.
+\ingroup correct
+\tparam Geometry \tparam_geometry
+\tparam Strategy \tparam_strategy{Area}
+\param geometry \param_geometry which will be corrected if necessary
+\param strategy \param_strategy{area}
+
+\qbk{distinguish,with strategy}
+
+\qbk{[include reference/algorithms/correct.qbk]}
+*/
+template <typename Geometry, typename Strategy>
+inline void correct(Geometry& geometry, Strategy const& strategy)
+{
+    resolve_variant::correct<Geometry>::apply(geometry, strategy);
 }
 
 #if defined(_MSC_VER)
