@@ -4,8 +4,8 @@
   Distributed under the Boost Software License, Version 1.0. (See accompanying
   file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
   =============================================================================*/
-#if !defined(SPIRIT_KEYWORDS_DETAIL_MARCH_13_2007_1145PM)
-#define SPIRIT_KEYWORDS_DETAIL_MARCH_13_2007_1145PM
+#ifndef BOOST_SPIRIT_REPOSITORY_QI_OPERATOR_DETAIL_KEYWORDS_HPP
+#define BOOST_SPIRIT_REPOSITORY_QI_OPERATOR_DETAIL_KEYWORDS_HPP
 
 #if defined(_MSC_VER)
 #pragma once
@@ -82,7 +82,9 @@ namespace boost { namespace spirit { namespace repository { namespace qi { names
                     Iterator save = first;
                     skipper_keyword_marker<Skipper,NoCasePass> 
                         marked_skipper(skipper,flags[Index::value],counters[Index::value]);
-                    if(subject.parse(first,last,context,marked_skipper,fusion::at_c<Index::value>(attr)))
+                    typename fusion::result_of::at_c<typename remove_reference<Attribute>::type, Index::value>::type
+                        attr_ = fusion::at_c<Index::value>(attr);
+                    if(subject.parse(first,last,context,marked_skipper,attr_))
                     {
                         return true;
                     }
@@ -188,7 +190,7 @@ namespace boost { namespace spirit { namespace repository { namespace qi { names
             ///////////////////////////////////////////////////////////////////////////
             // get_keyword_char_type
             //
-            // Collapses the character type comming from the subject kwd parsers and
+            // Collapses the character type coming from the subject kwd parsers and
             // and checks that they are all identical (necessary in order to be able
             // to build a tst parser to parse the keywords.
             ///////////////////////////////////////////////////////////////////////////
@@ -482,7 +484,7 @@ namespace boost { namespace spirit { namespace repository { namespace qi { names
             return true;
                     }
                     // Second pass case insensitive
-                    else if(parser_index_type* val_ptr
+                    if(parser_index_type* val_ptr
                             = lookup->find(saved_first,last,nc_filter()))
                     {
                         first = saved_first;
