@@ -26,19 +26,35 @@
 #include <boost/container/adaptive_pool.hpp>
 #include <boost/container/stable_vector.hpp>
 #include <boost/container/vector.hpp>
-#include <boost/timer/timer.hpp>
+#include <boost/move/detail/nsec_clock.hpp>
 
-using boost::timer::cpu_timer;
-using boost::timer::cpu_times;
-using boost::timer::nanosecond_type;
+using boost::move_detail::cpu_timer;
+using boost::move_detail::cpu_times;
+using boost::move_detail::nanosecond_type;
 
 namespace bc = boost::container;
 
-typedef std::allocator<int>   StdAllocator;
-typedef bc::allocator<int, 1> AllocatorPlusV1;
-typedef bc::allocator<int, 2> AllocatorPlusV2;
+class MyInt
+{
+   int int_;
+
+   public:
+   MyInt(int i = 0) : int_(i){}
+   MyInt(const MyInt &other)
+      :  int_(other.int_)
+   {}
+   MyInt & operator=(const MyInt &other)
+   {
+      int_ = other.int_;
+      return *this;
+   }
+};
+
+typedef std::allocator<MyInt>   StdAllocator;
+typedef bc::allocator<MyInt, 1> AllocatorPlusV1;
+typedef bc::allocator<MyInt, 2> AllocatorPlusV2;
 typedef bc::adaptive_pool
-   < int
+   < MyInt
    , bc::ADP_nodes_per_block
    , 0//bc::ADP_max_free_blocks
    , 2
@@ -58,27 +74,10 @@ template<> struct get_allocator_name<AllocatorPlusV2>
 template<> struct get_allocator_name<AdPool2PercentV2>
 {  static const char *get() {  return "AdPool2PercentV2";  } };
 
-class MyInt
-{
-   int int_;
-
-   public:
-   MyInt(int i = 0) : int_(i){}
-   MyInt(const MyInt &other)
-      :  int_(other.int_)
-   {}
-   MyInt & operator=(const MyInt &other)
-   {
-      int_ = other.int_;
-      return *this;
-   }
-};
-
 template<class Allocator>
 struct get_vector
 {
-   typedef bc::vector
-      <MyInt, typename Allocator::template rebind<MyInt>::other> type;
+   typedef bc::vector<MyInt, Allocator> type;
    static const char *vector_name()
    {
       return "vector<MyInt>";
@@ -89,7 +88,7 @@ template<class Allocator>
 struct get_stable_vector
 {
    typedef bc::stable_vector
-      <MyInt, typename Allocator::template rebind<MyInt>::other> type;
+      <MyInt, Allocator> type;
    static const char *vector_name()
    {
       return "stable_vector<MyInt>";
@@ -221,21 +220,25 @@ void stable_vector_operations()
 
 int main(int argc, const char *argv[])
 {
-   #define SINGLE_TEST
-   #ifndef SINGLE_TEST
-      #ifdef NDEBUG
-      unsigned int numit [] = { 40, 400, 4000, 40000 };
-      #else
-      unsigned int numit [] = { 4,   40,   400,   4000 };
-      #endif
-      unsigned int numele [] = { 10000, 1000, 100,   10     };
-   #else
+   //#define SINGLE_TEST
+   #define SIMPLE_IT
+   #ifdef SINGLE_TEST
       #ifdef NDEBUG
       unsigned int numit [] = { 40 };
       #else
       unsigned int numit [] = { 4 };
       #endif
       unsigned int numele [] = { 10000 };
+   #elif defined(SIMPLE_IT)
+      unsigned int numit [] = { 3 };
+      unsigned int numele [] = { 10000 };
+   #else
+      #ifdef NDEBUG
+      unsigned int numit [] = { 40, 400, 4000, 40000 };
+      #else
+      unsigned int numit [] = { 4,   40,   400,   4000 };
+      #endif
+      unsigned int numele [] = { 10000, 1000, 100,   10     };
    #endif
 
    //Warning: range erasure is buggy. Vector iterators are not stable, so it is not
