@@ -3,6 +3,10 @@
 
 // Copyright (c) 2010-2012 Barend Gehrels, Amsterdam, the Netherlands.
 
+// This file was modified by Oracle on 2020-2021.
+// Modifications copyright (c) 2020-2021, Oracle and/or its affiliates.
+// Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
+
 // Use, modification and distribution is subject to the Boost Software License,
 // Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
 // http://www.boost.org/LICENSE_1_0.txt)
@@ -16,28 +20,32 @@
 
 #include <boost/geometry/algorithms/detail/sections/sectionalize.hpp>
 #include <boost/geometry/algorithms/detail/sections/range_by_section.hpp>
-#include <boost/geometry/views/detail/range_type.hpp>
 #include <boost/geometry/geometries/geometries.hpp>
 #include <boost/geometry/geometries/point_xy.hpp>
 #include <boost/geometry/io/wkt/wkt.hpp>
+#include <boost/geometry/util/condition.hpp>
 
+// TEMP
+#include <boost/geometry/strategies/cartesian.hpp>
 
 
 template <int DimensionCount, bool Reverse, typename Geometry>
-void test_sectionalize(std::string const caseid, Geometry const& geometry, std::size_t section_count)
+void test_sectionalize(std::string const /*caseid*/, Geometry const& geometry, std::size_t section_count)
 {
     typedef typename bg::point_type<Geometry>::type point;
     typedef bg::model::box<point> box;
     typedef bg::sections<box, DimensionCount> sections;
 
+    typedef std::integer_sequence<std::size_t, 0> dim2;
+
     sections s;
-    bg::sectionalize<Reverse>(geometry, s);
+    bg::sectionalize<Reverse, dim2>(geometry, bg::detail::no_rescale_policy(), s);
 
     BOOST_CHECK_EQUAL(s.size(), section_count);
 
     typedef typename bg::closeable_view
         <
-            typename bg::detail::range_type<Geometry>::type const,
+            typename bg::ring_type<Geometry>::type const,
             bg::closure<Geometry>::value
         >::type cview_type;
     typedef typename bg::reversible_view
@@ -71,7 +79,7 @@ void test_sectionalize(std::string const& caseid, std::string const& wkt,
 {
     Geometry geometry;
     bg::read_wkt(wkt, geometry);
-    if (bg::closure<Geometry>::value == bg::open)
+    if ( BOOST_GEOMETRY_CONDITION( bg::closure<Geometry>::value == bg::open ) )
     {
         geometry.outer().resize(geometry.outer().size() - 1);
     }
