@@ -13,7 +13,7 @@
 #include <boost/container/vector.hpp>
 #include <boost/intrusive/detail/config_begin.hpp>
 #include "common_functors.hpp"
-#include <boost/detail/lightweight_test.hpp>
+#include <boost/core/lightweight_test.hpp>
 #include <boost/intrusive/options.hpp>
 #include <boost/intrusive/detail/iterator.hpp>
 #include "test_macros.hpp"
@@ -25,27 +25,22 @@ namespace boost{
 namespace intrusive{
 namespace test{
 
-template<class ValueTraits, class ContainerDefiner>
+template<class ContainerDefiner>
 struct test_generic_multiset
 {
-   typedef typename ValueTraits::value_type value_type;
-   typedef typename ValueTraits::pointer pointer;
-   typedef typename ValueTraits::const_pointer const_pointer;
-   typedef typename ValueContainer< value_type >::type value_cont_type;
-   typedef typename pointer_traits<pointer>::reference      reference;
-   typedef typename pointer_traits
-      <const_pointer>::reference                            const_reference;
+   typedef typename ContainerDefiner::value_cont_type    value_cont_type;
 
    static void test_all();
    static void test_sort(value_cont_type&);
    static void test_insert(value_cont_type&);
    static void test_swap(value_cont_type&);
+   static void test_merge(value_cont_type&);
    static void test_find(value_cont_type&);
    static void test_impl();
 };
 
-template<class ValueTraits, class ContainerDefiner>
-void test_generic_multiset<ValueTraits, ContainerDefiner>::test_all ()
+template<class ContainerDefiner>
+void test_generic_multiset<ContainerDefiner>::test_all ()
 {
    static const int random_init[6] = { 3, 2, 4, 1, 5, 2 };
    value_cont_type values (6);
@@ -69,14 +64,15 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_all ()
    test_sort(values);
    test_insert(values);
    test_swap(values);
+   test_merge(values);
    test_find(values);
    test_impl();
-   test_generic_assoc<ValueTraits, ContainerDefiner>::test_all(values);
+   test_generic_assoc<ContainerDefiner>::test_all(values);
 }
 
 //test case due to an error in tree implementation:
-template<class ValueTraits, class ContainerDefiner>
-void test_generic_multiset<ValueTraits, ContainerDefiner>::test_impl()
+template<class ContainerDefiner>
+void test_generic_multiset<ContainerDefiner>::test_impl()
 {
    value_cont_type values (5);
    for (int i = 0; i < 5; ++i)
@@ -97,8 +93,8 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_impl()
 }
 
 //test: constructor, iterator, clear, reverse_iterator, front, back, size:
-template<class ValueTraits, class ContainerDefiner>
-void test_generic_multiset<ValueTraits, ContainerDefiner>::test_sort(value_cont_type& values)
+template<class ContainerDefiner>
+void test_generic_multiset<ContainerDefiner>::test_sort(value_cont_type& values)
 {
    typedef typename ContainerDefiner::template container
       <>::type multiset_type;
@@ -123,14 +119,15 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_sort(value_cont_
 }
 
 //test: insert, const_iterator, const_reverse_iterator, erase, iterator_to:
-template<class ValueTraits, class ContainerDefiner>
-void test_generic_multiset<ValueTraits, ContainerDefiner>::test_insert(value_cont_type& values)
+template<class ContainerDefiner>
+void test_generic_multiset<ContainerDefiner>::test_insert(value_cont_type& values)
 {
    typedef typename ContainerDefiner::template container
       <>::type multiset_type;
 
    multiset_type testset;
    testset.insert(values.begin() + 2, values.begin() + 5);
+   testset.check();
    {  int init_values [] = { 1, 4, 5 };
       TEST_INTRUSIVE_SEQUENCE( init_values, testset.begin() );  }
 
@@ -138,6 +135,7 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_insert(value_con
    BOOST_TEST (i->value_ == 1);
 
    i = testset.insert (i, values[0]);
+   testset.check();
    BOOST_TEST (&*i == &values[0]);
 
    {  int init_values [] = { 5, 4, 3, 1 };
@@ -150,14 +148,15 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_insert(value_con
    BOOST_TEST (&*i == &values[2]);
 
    testset.erase(i);
+   testset.check();
 
    {  int init_values [] = { 1, 3, 5 };
       TEST_INTRUSIVE_SEQUENCE( init_values, testset.begin() );  }
 }
 
 //test: insert (seq-version), swap, erase (seq-version), size:
-template<class ValueTraits, class ContainerDefiner>
-void test_generic_multiset<ValueTraits, ContainerDefiner>::test_swap(value_cont_type& values)
+template<class ContainerDefiner>
+void test_generic_multiset<ContainerDefiner>::test_swap(value_cont_type& values)
 {
    typedef typename ContainerDefiner::template container
       <>::type multiset_type;
@@ -176,9 +175,57 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_swap(value_cont_
    BOOST_TEST (&*testset1.begin() == &values[3]);
 }
 
+template<class ContainerDefiner>
+void test_generic_multiset<ContainerDefiner>::test_merge(value_cont_type& values)
+{
+   typedef typename ContainerDefiner::template container
+      <>::type multiset_type;
+   typedef typename multiset_type::key_of_value key_of_value;
+   typedef typename multiset_type::key_type     key_type;
+
+   typedef typename ContainerDefiner::template container
+      < compare< std::greater<key_type> > >::type multiset_greater_type;
+
+   //original vector: 3, 2, 4, 1, 5, 2
+   //2,3
+   multiset_type testset1 (values.begin(), values.begin() + 2);
+   //5, 4, 2, 1
+   multiset_greater_type testset2;
+   testset2.insert (values.begin() + 2, values.begin() + 6);
+
+   testset2.merge(testset1);
+   testset1.check();
+   testset2.check();
+
+   BOOST_TEST (testset1.empty());
+   BOOST_TEST (testset2.size() == 6);
+   {  int init_values [] = { 5, 4, 3, 2, 2, 1 };
+      TEST_INTRUSIVE_SEQUENCE( init_values, testset2.begin() );  }
+
+   value_cont_type cmp_val_cont(1);
+   typename value_cont_type::reference cmp_val = cmp_val_cont.front();
+   (&cmp_val)->value_ = 2;
+
+   BOOST_TEST (*testset2.find(key_of_value()(cmp_val)) == values[5]);
+   BOOST_TEST (*testset2.find(2, any_greater()) == values[5]);
+   BOOST_TEST (&*(++testset2.find(key_of_value()(cmp_val))) == &values[1]);
+
+   testset1.merge(testset2);
+   testset1.check();
+   testset2.check();
+
+   BOOST_TEST (testset1.size() == 6);
+   {  int init_values [] = { 1, 2, 2, 3, 4, 5 };
+      TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
+   BOOST_TEST (*testset1.find(key_of_value()(cmp_val)) == values[5]);
+   BOOST_TEST (*testset1.find(2, any_less()) == values[5]);
+   BOOST_TEST (&*(++testset1.find(key_of_value()(cmp_val))) == &values[1]);
+   BOOST_TEST (testset2.empty());
+}
+
 //test: find, equal_range (lower_bound, upper_bound):
-template<class ValueTraits, class ContainerDefiner>
-void test_generic_multiset<ValueTraits, ContainerDefiner>::test_find(value_cont_type& values)
+template<class ContainerDefiner>
+void test_generic_multiset<ContainerDefiner>::test_find(value_cont_type& values)
 {
    typedef typename ContainerDefiner::template container
       <>::type multiset_type;
@@ -189,12 +236,14 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_find(value_cont_
 
    {
       value_cont_type cmp_val_cont(1);
-      reference cmp_val = cmp_val_cont.front();
+      typename value_cont_type::reference cmp_val = cmp_val_cont.front();
       (&cmp_val)->value_ = 2;
       iterator i = testset.find (key_of_value()(cmp_val));
+      BOOST_TEST (i == testset.find (2, any_less()));
       BOOST_TEST (i->value_ == 2);
       BOOST_TEST ((++i)->value_ == 2);
       std::pair<iterator,iterator> range = testset.equal_range (key_of_value()(cmp_val));
+      BOOST_TEST(range == testset.equal_range (2, any_less()));
 
       BOOST_TEST (range.first->value_ == 2);
       BOOST_TEST (range.second->value_ == 3);
@@ -202,19 +251,21 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_find(value_cont_
 
       (&cmp_val)->value_ = 7;
       BOOST_TEST (testset.find(key_of_value()(cmp_val)) == testset.end());
+      BOOST_TEST (testset.find (7, any_less()) == testset.end());
    }
    {  //1, 2, 2, 3, 4, 5
       const multiset_type &const_testset = testset;
       std::pair<iterator,iterator> range;
       std::pair<const_iterator, const_iterator> const_range;
       value_cont_type cmp_val_cont(2);
-      reference cmp_val_lower = cmp_val_cont.front();
-      reference cmp_val_upper = cmp_val_cont.back();
+      typename value_cont_type::reference cmp_val_lower = cmp_val_cont.front();
+      typename value_cont_type::reference cmp_val_upper = cmp_val_cont.back();
       {
       (&cmp_val_lower)->value_ = 1;
       (&cmp_val_upper)->value_ = 2;
       //left-closed, right-closed
       range = testset.bounded_range (key_of_value()(cmp_val_lower), key_of_value()(cmp_val_upper), true, true);
+      BOOST_TEST (range == testset.bounded_range (1, 2, any_less(), true, true));
       BOOST_TEST (range.first->value_ == 1);
       BOOST_TEST (range.second->value_ == 3);
       BOOST_TEST (boost::intrusive::iterator_distance (range.first, range.second) == 3);
@@ -223,6 +274,7 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_find(value_cont_
       (&cmp_val_lower)->value_ = 1;
       (&cmp_val_upper)->value_ = 2;
       const_range = const_testset.bounded_range (key_of_value()(cmp_val_lower), key_of_value()(cmp_val_upper), true, false);
+      BOOST_TEST (const_range == const_testset.bounded_range (1, 2, any_less(), true, false));
       BOOST_TEST (const_range.first->value_ == 1);
       BOOST_TEST (const_range.second->value_ == 2);
       BOOST_TEST (boost::intrusive::iterator_distance (const_range.first, const_range.second) == 1);
@@ -230,6 +282,7 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_find(value_cont_
       (&cmp_val_lower)->value_ = 1;
       (&cmp_val_upper)->value_ = 3;
       range = testset.bounded_range (key_of_value()(cmp_val_lower), key_of_value()(cmp_val_upper), true, false);
+      BOOST_TEST (range == testset.bounded_range (1, 3, any_less(), true, false));
       BOOST_TEST (range.first->value_ == 1);
       BOOST_TEST (range.second->value_ == 3);
       BOOST_TEST (boost::intrusive::iterator_distance (range.first, range.second) == 3);
@@ -238,6 +291,7 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_find(value_cont_
       (&cmp_val_lower)->value_ = 1;
       (&cmp_val_upper)->value_ = 2;
       const_range = const_testset.bounded_range (key_of_value()(cmp_val_lower), key_of_value()(cmp_val_upper), false, true);
+      BOOST_TEST (const_range == const_testset.bounded_range (1, 2, any_less(), false, true));
       BOOST_TEST (const_range.first->value_ == 2);
       BOOST_TEST (const_range.second->value_ == 3);
       BOOST_TEST (boost::intrusive::iterator_distance (const_range.first, const_range.second) == 2);
@@ -246,6 +300,7 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_find(value_cont_
       (&cmp_val_lower)->value_ = 1;
       (&cmp_val_upper)->value_ = 2;
       range = testset.bounded_range (key_of_value()(cmp_val_lower), key_of_value()(cmp_val_upper), false, false);
+      BOOST_TEST (range == testset.bounded_range (1, 2, any_less(), false, false));
       BOOST_TEST (range.first->value_ == 2);
       BOOST_TEST (range.second->value_ == 2);
       BOOST_TEST (boost::intrusive::iterator_distance (range.first, range.second) == 0);
@@ -254,6 +309,7 @@ void test_generic_multiset<ValueTraits, ContainerDefiner>::test_find(value_cont_
       (&cmp_val_lower)->value_ = 5;
       (&cmp_val_upper)->value_ = 6;
       const_range = const_testset.bounded_range (key_of_value()(cmp_val_lower), key_of_value()(cmp_val_upper), true, false);
+      BOOST_TEST (const_range == const_testset.bounded_range (5, 6, any_less(), true, false));
       BOOST_TEST (const_range.first->value_ == 5);
       BOOST_TEST (const_range.second == const_testset.end());
       BOOST_TEST (boost::intrusive::iterator_distance (const_range.first, const_range.second) == 1);
