@@ -20,9 +20,8 @@
 #include <boost/assert.hpp>
 #include <boost/cstdint.hpp>
 #include <boost/throw_exception.hpp>
-#include <boost/system/error_code.hpp>
-#include <boost/system/system_error.hpp>
 #include <boost/log/detail/event.hpp>
+#include <boost/log/exceptions.hpp>
 
 #if defined(BOOST_LOG_EVENT_USE_FUTEX)
 
@@ -35,8 +34,13 @@
 // Some Android NDKs (Google NDK and older Crystax.NET NDK versions) don't define SYS_futex
 #if defined(SYS_futex)
 #define BOOST_LOG_SYS_FUTEX SYS_futex
-#else
+#elif defined(__NR_futex)
 #define BOOST_LOG_SYS_FUTEX __NR_futex
+// riscv32 defines a different system call instead of __NR_futex
+#elif defined(__NR_futex_time64)
+#define BOOST_LOG_SYS_FUTEX __NR_futex_time64
+#else
+#error "Unable to find a suitable futex"
 #endif
 
 #if defined(FUTEX_WAIT_PRIVATE)
@@ -96,7 +100,7 @@ BOOST_LOG_API void futex_based_event::wait()
     {
         while (true)
         {
-            if (::syscall(BOOST_LOG_SYS_FUTEX, &m_state.storage(), BOOST_LOG_FUTEX_WAIT, 0, NULL, NULL, 0) == 0)
+            if (::syscall(BOOST_LOG_SYS_FUTEX, &m_state.value(), BOOST_LOG_FUTEX_WAIT, 0, NULL, NULL, 0) == 0)
             {
                 // Another thread has set the event while sleeping
                 break;
@@ -110,8 +114,7 @@ BOOST_LOG_API void futex_based_event::wait()
             }
             else if (BOOST_UNLIKELY(err != EINTR))
             {
-                BOOST_THROW_EXCEPTION(system::system_error(
-                    err, system::system_category(), "Failed to block on the futex"));
+                BOOST_LOG_THROW_DESCR_PARAMS(system_error, "Failed to block on the futex", (err));
             }
         }
 
@@ -124,11 +127,10 @@ BOOST_LOG_API void futex_based_event::set_signalled()
 {
     if (m_state.exchange(1, boost::memory_order_release) == 0)
     {
-        if (BOOST_UNLIKELY(::syscall(BOOST_LOG_SYS_FUTEX, &m_state.storage(), BOOST_LOG_FUTEX_WAKE, 1, NULL, NULL, 0) < 0))
+        if (BOOST_UNLIKELY(::syscall(BOOST_LOG_SYS_FUTEX, &m_state.value(), BOOST_LOG_FUTEX_WAKE, 1, NULL, NULL, 0) < 0))
         {
             const int err = errno;
-            BOOST_THROW_EXCEPTION(system::system_error(
-                err, system::system_category(), "Failed to wake threads blocked on the futex"));
+            BOOST_LOG_THROW_DESCR_PARAMS(system_error, "Failed to wake threads blocked on the futex", (err));
         }
     }
 }
@@ -141,8 +143,7 @@ BOOST_LOG_API sem_based_event::sem_based_event() : m_state()
     if (BOOST_UNLIKELY(sem_init(&m_semaphore, 0, 0) != 0))
     {
         const int err = errno;
-        BOOST_THROW_EXCEPTION(system::system_error(
-            err, system::system_category(), "Failed to initialize semaphore"));
+        BOOST_LOG_THROW_DESCR_PARAMS(system_error, "Failed to initialize semaphore", (err));
     }
 }
 
@@ -163,8 +164,7 @@ BOOST_LOG_API void sem_based_event::wait()
             const int err = errno;
             if (BOOST_UNLIKELY(err != EINTR))
             {
-                BOOST_THROW_EXCEPTION(system::system_error(
-                    err, system::system_category(), "Failed to block on the semaphore"));
+                BOOST_LOG_THROW_DESCR_PARAMS(system_error, "Failed to block on the semaphore", (err));
             }
         }
         else
@@ -181,8 +181,7 @@ BOOST_LOG_API void sem_based_event::set_signalled()
         if (BOOST_UNLIKELY(sem_post(&m_semaphore) != 0))
         {
             const int err = errno;
-            BOOST_THROW_EXCEPTION(system::system_error(
-                err, system::system_category(), "Failed to wake the blocked thread"));
+            BOOST_LOG_THROW_DESCR_PARAMS(system_error, "Failed to wake the blocked thread", (err));
         }
     }
 }
@@ -197,8 +196,7 @@ BOOST_LOG_API winapi_based_event::winapi_based_event() :
     if (BOOST_UNLIKELY(!m_event))
     {
         const DWORD err = GetLastError();
-        BOOST_THROW_EXCEPTION(system::system_error(
-            err, system::system_category(), "Failed to create Windows event"));
+        BOOST_LOG_THROW_DESCR_PARAMS(system_error, "Failed to create Windows event", (err));
     }
 }
 
@@ -217,8 +215,7 @@ BOOST_LOG_API void winapi_based_event::wait()
         if (BOOST_UNLIKELY(WaitForSingleObject(m_event, INFINITE) != 0))
         {
             const DWORD err = GetLastError();
-            BOOST_THROW_EXCEPTION(system::system_error(
-                err, system::system_category(), "Failed to block on Windows event"));
+            BOOST_LOG_THROW_DESCR_PARAMS(system_error, "Failed to block on Windows event", (err));
         }
     }
     const_cast< volatile boost::uint32_t& >(m_state) = 0;
@@ -233,8 +230,7 @@ BOOST_LOG_API void winapi_based_event::set_signalled()
         {
             const DWORD err = GetLastError();
             const_cast< volatile boost::uint32_t& >(m_state) = 0;
-            BOOST_THROW_EXCEPTION(system::system_error(
-                err, system::system_category(), "Failed to wake the blocked thread"));
+            BOOST_LOG_THROW_DESCR_PARAMS(system_error, "Failed to wake the blocked thread", (err));
         }
     }
 }
