@@ -11,6 +11,11 @@
 #include <iostream>
 #include <boost/detail/lightweight_test.hpp>
 
+#ifdef _MSC_VER
+// bogus https://developercommunity.visualstudio.com/t/buggy-warning-c4709/471956
+# pragma warning(disable: 4709) // comma operator within array index expression
+#endif
+
 using namespace std;
 using namespace BOOST_SPIRIT_CLASSIC_NS;
 
