@@ -8,14 +8,14 @@
 // if a token matched at the beginning of a line is discarded using 
 // lex::pass_fail.
 
-// #define BOOST_SPIRIT_LEXERTL_DEBUG 1
-
 #include <boost/config/warning_disable.hpp>
 #include <boost/detail/lightweight_test.hpp>
 
 #include <boost/spirit/include/support_multi_pass.hpp>
 #include <boost/spirit/include/classic_position_iterator.hpp>
 #include <boost/spirit/include/lex_lexertl.hpp>
+#include <boost/phoenix/operator/self.hpp>
+#include <boost/phoenix/statement/sequence.hpp>
 
 namespace spirit = boost::spirit;
 namespace lex = spirit::lex;
@@ -44,20 +44,20 @@ struct lexer
         self =  word [ 
                     lex::_state = "O" 
                 ]
-            |   lex::token_def<>("!.*$") [ 
+            |   lex::token_def<>("!.*$") [(
                     lex::_state = "O"
                   , lex::_pass = lex::pass_flags::pass_ignore 
-                ]
+                )]
             |   lex::token_def<>('\n', 2) [ 
                     lex::_state = "O" 
                 ] 
             ;
         
         self("O") = 
-                lex::token_def<>(".") [ 
+                lex::token_def<>(".") [(
                     lex::_state = "INITIAL"
                   , lex::_pass = lex::pass_flags::pass_fail 
-                ]
+                )]
             ;
     }
     
